# Le design pattern parcours d'un arbre sans erreur en python

![category](développement)
![tag](designpattern)
![tag](python)

Le dictionnaire et la liste sont des objets de base très puissant en python.

Mais souvent on peut être bloqué par des erreurs, et avoir une lecture un peu difficile

```python
durand = {"name":"Durand", "emails":[{'email':"durand@mail.com"}]}
dupont = {"name":"Dupont"}

persons = []
persons.append(durand)
persons.append(dupont)

for person in persons:
    print(person['name'], person['emails'][0]['email'])
```

```bash
>>> Traceback (most recent call last):
>>>   File "test.py", line 40, in <module>
>>>     print(person.name, person.emails[0].email)
>>> AttributeError: 'dict' object has no attribute 'name'
```

On obtient une erreur car dupont n'a pas de segments *emails* et en plus le synthaxe est compliqué:

```python
person['emails'][0]['email']
```

On préferait avoir une synthaxe du type

```python
person.emails.[0].email
# ou mieux
person.emails.[0].email
```

Il est possible de faire cela en créant

- une liste générique gérant l'usage ou non d'un index (emails[0].email ou emails.email)
- un objet generic permettant de transformer un dictionnaire en un objet
- un objet spécifique qui répond toujours quand on l'appelle quelque soit le moyen (attribut ou index)

cela donne

```python
import collections


class returnStillNothing(str):

    def __getattr__(self, id):
        return returnStillNothing()

    def __getitem__(self, i):
        return returnStillNothing()

    def __call__(self, *args, **kw):
        return returnStillNothing()


class GenericList(collections.UserList):

    def __init__(self):
        collections.UserList.__init__(self)

    def __getattr__(self, id):
        if len(self) > 0:
            return getattr(self[0], id)
        return returnStillNothing()


class GenericModel():

    def __init__(self, **kw):
        for key in kw:
            self.__setattr__(key, kw[key])

    def __getattr__(self, id):
        return returnStillNothing()


durand = GenericModel(**{"name":"Durand"})
durand.emails = GenericList()
durand.emails.append(GenericModel(**{'email':"durand@mail.com"}))
dupont = GenericModel(**{"name":"Dupont"})
persons = GenericList()
persons.append(durand)
persons.append(dupont)

for person in persons:
    print(person.name, person.emails[0].email, person.emails.email, person.yyy.xxx)

```

plus d'erreur lecture simple

```note
il est possible de modifier GenericModel afin d'ajouter dans getattr des fonctions permettant de gérer les sous niveau.

J'utilise ce système avec l'usage de SGBD ou d'API ... si j'ai un attribut spécifique je peux faire un appel SGBD ou API pour chercher l'information
```
